<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html" />

    <!-- define a variable for the value of a non breaking space in the html output -->
  <xsl:variable name="nbsp" select="'&#160;'" />

  <!-- process the root of the XML trace file (this represents all data from a trace file) -->
  <xsl:template match="/">
    <html>
      <head>
        <script type="text/javascript" src="BrowserTraceDisplay.js" />
        <style>
          pre.stack { margin-top: 0px; }
          
          /* attributes specific to banner lines */
          div.bannerline         { display: inline; float: left; text-align: left; }
          
          /* attributes specific to trace header values */
          div.headingline        { width:  50em; display: inline; float: left; text-align: left; margin-left: 5em; text-decoration: underline; }
          div.headlabel          { width:  15em; display: inline; float: left; text-align: right; margin-left: 2em; }
          div.headvalue          { width:  20em; display: inline; float: left; margin-left: 2em; }
          div.timevalue          { width:   6.3em; display: inline; float: left; text-align: right; margin-left: 2em; }
          div.traceindex         { width:  10em; display: inline; float: left; }

          /* attributes specific to entry fields */
          div.expander           { width:   1em; display: inline; float: left; margin-left: 1em; }
          img.expander           { width:   8px; height: 8px; text-align: top; border: 1px solid #0000ff; }
          div.calllevel          { width:   2em; display: inline; float: left; text-align: right; }
          div.exitnumber         { width:   2em; display: inline; float: left; text-align: right; }
          div.method             { width:  35em; display: inline; float: left; text-align: right; margin-left: 10px; margin-right: 10px; }
          div.methodclipped      { width:  35em; display: inline; float: left; text-align: right; cursor: pointer; margin-left: 10px; margin-right: 10px; }
          div.entertime          { width:  10em; display: inline; float: left; text-align: right; }
          div.timedelta          { width:   7em; display: inline; float: left; text-align: right; margin-left: 20px; }
          div.arguments          { width: 100em; display: inline; float: left; margin-left: 20px; overflow: visible; }
        </style>
      </head>
      <body>
        <xsl:apply-templates />
      </body>
    </html>
  </xsl:template>

  <!-- process the trace entries (this tag represents the full display of a single trace buffer) -->
  <xsl:template match="trace-display">
    <br/><br/><br/>
    <div style="width=200em;">
      <div class="expander" style="cursor:pointer" onclick="toggleBlock(this)">
          <img class="expander" src="Minus.gif" />
      </div>
      <div class="bannerline"><xsl:value-of select="@message"/></div>
      <div>
      <br/><br/>
        <xsl:apply-templates />
      </div>
    </div>
  </xsl:template>

  <!-- process the trace heading (this tag represents data used to manage the trace buffer) -->
  <xsl:template match="trace-heading">
    <div>
      <br/>
      <div class="headingline">Trace Buffer Identification</div>
      <br/>
    </div>
    <div>
      <br/>
      <div class="headlabel">thread-id:</div>
      <div class="headvalue"><xsl:value-of select="@thread-id"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">thread-name:</div>
      <div class="headvalue"><xsl:value-of select="@thread-name"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">current-date-time:</div>
      <div class="headvalue"><xsl:value-of select="@current-date-time"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">start-date-time:</div>
      <div class="headvalue"><xsl:value-of select="@start-date-time"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">date-time-terminated:</div>
      <div class="headvalue"><xsl:value-of select="@date-time-terminated"/></div>
      <br/>
    </div>
    <br/>
    <xsl:apply-templates />
    <div>
      <br/>
      <div class="headingline">Trace Buffer</div>
      <br/>
    </div>
    <div style="width: 200em;">
      <br/>
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div class="calllevel">Cl</div>
      <div class="exitnumber">Ex</div>
      <div class="method">Plug-in, Class, and Method Name</div>
      <div class="entertime">Time Enter</div>
      <div class="timedelta">Time Delta</div>
      <div class="arguments">(arguments) [return value]</div>
      <br/>
    </div>
    <div style="width: 200em;">
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div class="calllevel">Lv</div>
      <div class="exitnumber">Nb</div>
      <div class="method">Plugin.&lt;Class&gt;.Method</div>
      <div class="entertime">(sec.usec)</div>
      <div class="timedelta">(usec)</div>
      <div class="arguments"> </div>
      <br/>
    </div>
    <div style="width: 200em;">
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div>-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------</div>
    </div>
  </xsl:template>

  <!-- process the time values (when traces were started, stopped, newest and oldest trace time stamps) -->
  <xsl:template match="time-values">
    <div>
      <div class="headlabel">start-time:</div>
      <div class="timevalue"><xsl:value-of select="@start-time"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">current-time:</div>
      <div class="timevalue"><xsl:value-of select="@current-time"/></div>
      <br/>
    </div>
    <br/>
    <div>
      <div class="headlabel">newest-trace:</div>
      <div class="timevalue"><xsl:value-of select="@newest-trace"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">oldest-trace:</div>
      <div class="timevalue" style="text-decoration: underline;"><xsl:value-of select="@oldest-trace"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">time-delta:</div>
      <div class="timevalue"><xsl:value-of select="@time-delta"/></div>
      <br/>
    </div>
  </xsl:template>

  <!-- process the trace control data (this tag represents the data that controls use of the trace buffer) -->
  <xsl:template match="trace-control">
    <div>
      <br/>
      <div class="headingline">Trace Buffer Control Data</div>
      <br/>
    </div>
    <div>
      <br/>
      <div class="headlabel">version:</div>
      <div class="headvalue"><xsl:value-of select="@version"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">os:</div>
      <div class="headvalue"><xsl:value-of select="@os"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">state:</div>
      <div class="headvalue"><xsl:value-of select="@state"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">subState:</div>
      <div class="headvalue"><xsl:value-of select="@subState"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">trace-count:</div>
      <div class="headvalue"><xsl:value-of select="@trace-count"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">trace-index:</div>
      <div class="headvalue"><xsl:value-of select="@trace-index"/></div>
      <br/>
    </div>
    <div>
      <div class="headlabel">call-level:</div>
      <div class="headvalue"><xsl:value-of select="@call-level"/></div>
      <br/>
    </div>
    <br/>
  </xsl:template>

  <!-- process the call stack -->
  <xsl:template match="call-stack">
    <div>
      <br/>
      <div class="headingline">Trace Buffer Call Stack</div>
      <br/>
    </div>
    <br/>
    <div style="width: 200em;">
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div class="calllevel">Cl</div>
      <div class="method">Plug-in, Class, and Method Name</div>
      <div class="entertime">Time Enter</div>
      <div class="traceindex">Trace Index</div>
      <br/>
    </div>
    <div style="width: 200em; ">
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div class="calllevel">Lv</div>
      <div class="method">Plugin.&lt;Class&gt;.Method</div>
      <div class="entertime">(sec.usec)</div>
      <div class="traceindex"></div>
      <br/>
    </div>
    <div style="width: 200em;">
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div>---------------------------------------------------------------------------------------------------------------------------------------------</div>
    </div>
    <xsl:apply-templates />
  </xsl:template>

  <!-- process the entries in the call stack -->
  <xsl:template match="call-stack-entry">
    <div>
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div class="calllevel"><xsl:value-of select="@call-level"/></div>
      <xsl:call-template name="format-method-name">
        <xsl:with-param name="method-name" select="@method-name" />
      </xsl:call-template>
      <div class="entertime"><xsl:value-of select="@enter-time"/></div>
      <div class="traceindex"><xsl:value-of select="@trace-entry-index"/></div>
      <br/>
    </div>
  </xsl:template>

  <!-- process the ascii trace entries (this tag represents an ascii trace entry) -->
  <xsl:template match="entry-ascii">
    <div style="width: 200em;">
      <div class="expander"><xsl:value-of select="$nbsp"/></div>
      <div class="calllevel" style="text-align: left;">A</div>
      <div class="exitnumber"><xsl:value-of select="$nbsp"/></div>
      <xsl:call-template name="format-method-name">
        <xsl:with-param name="method-name" select="@trace-label" />
      </xsl:call-template>
      <div class="entertime"><xsl:value-of select="@enter-time"/></div>
      <div class="timedelta"><xsl:value-of select="$nbsp"/></div>
      <div class="arguments"><xsl:value-of select="@trace-message"/></div>
      <br/>
    </div>
  </xsl:template>

  <!-- process the call trace entries (this tag represents a method entry and exit trace entry) -->
  <xsl:template match="entry-call">
    <xsl:apply-templates />
  </xsl:template>

  <!-- process call entry elements that have a subordinate entry (need expand/collapse controls) -->
  <xsl:template match="entry-call[entry-call]">
    <xsl:call-template name="format-entry-call">
      <xsl:with-param name="expander" select="'-'" />
    </xsl:call-template>
  </xsl:template>

  <!-- process call entry elements that do not have a subordinate entry (no expand/collapse controls) -->
  <xsl:template match="entry-call[not(entry-call)]">
    <xsl:call-template name="format-entry-call">
      <xsl:with-param name="expander" select="' '" />
    </xsl:call-template>
  </xsl:template>

  <!-- process the call trace entries (this tag represents a method entry and exit trace entry) -->
  <xsl:template match="exception">
    <div style="width: 200em; color:red; ">
      <div class="expander" style="cursor:pointer" onclick="toggleBlock(this)">
        <img class="expander" src="Plus.gif" />
      </div>
      <div class="calllevel" style="text-align: left;">X</div>
      <div class="exitnumber"><xsl:value-of select="$nbsp"/></div>
      <xsl:call-template name="format-method-name">
        <xsl:with-param name="method-name" select="@class" />
      </xsl:call-template>
      <div class="entertime"><xsl:value-of select="$nbsp"/></div>
      <div class="timedelta"><xsl:value-of select="$nbsp"/></div>
      <div class="arguments"><xsl:value-of select="@message"/></div>
      <div style="display: none; text-align: left;" >
        <br/>
        <pre class="stack">
          <xsl:value-of select="."/>
        </pre>
      </div>
      <br/>
    </div>
  </xsl:template>

  <!-- format the method name field in a trace entry -->
  <xsl:template name="format-method-name">
    <xsl:param name="method-name"></xsl:param>
      <xsl:choose>
        <xsl:when test="string-length($method-name) = 0" >
          <div class="method"><xsl:value-of select="$nbsp"/></div>
        </xsl:when>
        <xsl:when test="string-length($method-name) >= 85" >
          <xsl:element name="div">
            <xsl:attribute name="title">
              <xsl:value-of select="$method-name" />
            </xsl:attribute>
            <div class="methodclipped"><xsl:value-of select="concat('*', substring($method-name,string-length($method-name)-84,85))"/></div>
          </xsl:element>
        </xsl:when>
        <xsl:otherwise>
          <div class="method"><xsl:value-of select="$method-name"/></div>
        </xsl:otherwise>
      </xsl:choose>
  </xsl:template>

  <!-- format the HTML to display a single trace entry in a browser -->
  <xsl:template name="format-entry-call">
    <xsl:param name="expander"></xsl:param>
    <!-- format the html that contains the trace entry content -->
    <div style="width: 200em;">
      <xsl:choose>
        <xsl:when test="contains($expander, '-')" >
          <div class="expander" style="cursor:pointer" onclick="toggleBlock(this)">
            <img class="expander" src="Minus.gif" />
          </div>
        </xsl:when>
        <xsl:when test="contains($expander, '+')" >
          <div class="expander" style="cursor:pointer" onclick="toggleBlock(this)">
            <img class="expander" src="Plus.gif" />
          </div>
        </xsl:when>
        <xsl:otherwise>
          <div class="expander"><xsl:value-of select="$nbsp"/></div>
        </xsl:otherwise>
      </xsl:choose>
      <div class="calllevel"><xsl:value-of select="@call-level"/></div>
      <div class="exitnumber"><xsl:value-of select="@exit-number"/></div>
      <xsl:call-template name="format-method-name">
        <xsl:with-param name="method-name" select="@method-name" />
      </xsl:call-template>
      <div class="entertime"><xsl:value-of select="@enter-time"/></div>
      <div class="timedelta"><xsl:value-of select="@time-delta"/></div>
      <div class="arguments"><xsl:value-of select="concat(@arguments, @return-value)"/></div>
      <br/>
      <div>
        <!-- process any subornate entries in the xml (trace entries with a higher call level) -->
        <xsl:apply-templates />
      </div>
    </div>
  </xsl:template>
</xsl:stylesheet>